@@ -23,74 +23,13 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdarg.h>
-#include <base64.h>
+#include <arpa/inet.h>
+#include <base64-helper.h>
 
-#include <vpn.h>
 #include <worker.h>
 #include <main.h>
 #include <sec-mod.h>
 
-char *human_addr2(const struct sockaddr *sa, socklen_t salen,
-		       void *_buf, size_t buflen, unsigned full)
-{
-	char *save_buf = _buf;
-	char *buf = _buf;
-	size_t l;
-
-	if (!buf || !buflen)
-		return NULL;
-
-	if (sa->sa_family != AF_INET && sa->sa_family != AF_INET6) {
-		return NULL;
-	}
-
-	if (full != 0 && salen == sizeof(struct sockaddr_in6) &&
-		((struct sockaddr_in6*)sa)->sin6_port != 0) {
-		*buf = '[';
-		buf++;
-		buflen--;
-	}
-
-	if (getnameinfo(sa, salen, buf, buflen, NULL, 0, NI_NUMERICHOST) != 0)
-		return NULL;
-
-	if (salen == sizeof(struct sockaddr_in6)) {
-		char *p = strchr(buf, '%');
-		/* remove any zone info */
-		if (p != NULL) {
-			*p = 0;
-		}
-	}
-
-	if (full == 0)
-		goto finish;
-
-	l = strlen(buf);
-	buf += l;
-	buflen -= l;
-
-	if (salen == sizeof(struct sockaddr_in6) &&
-		((struct sockaddr_in6*)sa)->sin6_port != 0) {
-		*buf = ']';
-		buf++;
-		buflen--;
-	}
-
-	*buf = ':';
-	buf++;
-	buflen--;
-
-	if (getnameinfo(sa, salen, NULL, 0, buf, buflen, NI_NUMERICSERV) != 0)
-		return NULL;
-
-	if (buf[0] == '0' && buf[1] == 0) {
-		buf--;
-		buf[0] = 0;
-	}
-
-finish:
-	return save_buf;
-}
 
 void __attribute__ ((format(printf, 3, 4)))
     _oclog(const worker_st * ws, int priority, const char *fmt, ...)
@@ -99,16 +38,21 @@ void __attribute__ ((format(printf, 3, 4)))
 	const char* ip;
 	va_list args;
 
-	if (priority == LOG_DEBUG && ws->config->debug < 3)
+	if (priority == LOG_DEBUG && ws->perm_config->debug < DEBUG_INFO)
 		return;
 
 	if (priority == LOG_HTTP_DEBUG) {
-	    if (ws->config->debug < DEBUG_HTTP)
+	    if (ws->perm_config->debug < DEBUG_HTTP)
                 return;
             else
                 priority = LOG_INFO;
         } else if (priority == LOG_TRANSFER_DEBUG) {
-	    if (ws->config->debug < DEBUG_TRANSFERRED)
+	    if (ws->perm_config->debug < DEBUG_TRANSFERRED)
+                return;
+            else
+                priority = LOG_DEBUG;
+        } else if (priority == LOG_SENSITIVE) {
+	    if (ws->perm_config->debug < DEBUG_SENSITIVE)
                 return;
             else
                 priority = LOG_DEBUG;
@@ -142,16 +86,16 @@ void __attribute__ ((format(printf, 4, 5)))
 	const char* ip = NULL;
 	va_list args;
 
-	if (priority == LOG_DEBUG && s->config->debug < 3)
+	if (priority == LOG_DEBUG && s->perm_config->debug < 3)
 		return;
 
 	if (priority == LOG_HTTP_DEBUG) {
-	    if (s->config->debug < DEBUG_HTTP)
+	    if (s->perm_config->debug < DEBUG_HTTP)
                 return;
             else
                 priority = LOG_DEBUG;
         } else if (priority == LOG_TRANSFER_DEBUG) {
-	    if (s->config->debug < DEBUG_TRANSFERRED)
+	    if (s->perm_config->debug < DEBUG_TRANSFERRED)
                 return;
             else
                 priority = LOG_DEBUG;
@@ -186,11 +130,11 @@ void  mslog_hex(const main_server_st * s, const struct proc_st* proc,
 	size_t buf_size;
 	gnutls_datum_t data = {bin, bin_size};
 
-	if (priority == LOG_DEBUG && s->config->debug == 0)
+	if (priority == LOG_DEBUG && s->perm_config->debug == 0)
 		return;
 
 	if (b64) {
-		base64_encode((char*)bin, bin_size, (char*)buf, sizeof(buf));
+		oc_base64_encode((char*)bin, bin_size, (char*)buf, sizeof(buf));
 	} else {
 		buf_size = sizeof(buf);
 		ret = gnutls_hex_encode(&data, buf, &buf_size);
@@ -211,11 +155,11 @@ void  oclog_hex(const worker_st* ws, int priority,
 	size_t buf_size;
 	gnutls_datum_t data = {bin, bin_size};
 
-	if (priority == LOG_DEBUG && ws->config->debug == 0)
+	if (priority == LOG_DEBUG && ws->perm_config->debug == 0)
 		return;
 
 	if (b64) {
-		base64_encode((char*)bin, bin_size, (char*)buf, sizeof(buf));
+		oc_base64_encode((char*)bin, bin_size, (char*)buf, sizeof(buf));
 	} else {
 		buf_size = sizeof(buf);
 		ret = gnutls_hex_encode(&data, buf, &buf_size);
@@ -236,11 +180,11 @@ void  seclog_hex(const struct sec_mod_st* sec, int priority,
 	size_t buf_size;
 	gnutls_datum_t data = {bin, bin_size};
 
-	if (priority == LOG_DEBUG && sec->config->debug == 0)
+	if (priority == LOG_DEBUG && sec->perm_config->debug == 0)
 		return;
 
 	if (b64) {
-		base64_encode((char*)bin, bin_size, (char*)buf, sizeof(buf));
+		oc_base64_encode((char*)bin, bin_size, (char*)buf, sizeof(buf));
 	} else {
 		buf_size = sizeof(buf);
 		ret = gnutls_hex_encode(&data, buf, &buf_size);
