@@ -360,9 +360,14 @@ int handle_worker_commands(main_server_st * s, struct proc_st *proc)
 			if (tmsg->dtls_compr)
 				strlcpy(proc->dtls_compr, tmsg->dtls_compr,
 					 sizeof(proc->dtls_compr));
-			if (tmsg->user_agent)
+
+			if (tmsg->user_agent && tmsg->device_type == NULL)
 				strlcpy(proc->user_agent, tmsg->user_agent,
 					 sizeof(proc->user_agent));
+			else if (tmsg->user_agent && tmsg->device_type)
+				snprintf(proc->user_agent, sizeof(proc->user_agent), "%s / %s",
+					 tmsg->user_agent, tmsg->device_type);
+
 			if (tmsg->hostname) {
 				strlcpy(proc->hostname, tmsg->hostname,
 					 sizeof(proc->hostname));
